import Vapor

struct CategoriesController: RouteCollection {
    func boot(routes: RoutesBuilder) throws {
        let categoriesRoute = routes.grouped("api", "categories")
        categoriesRoute.get(use: getAllHandler)
        categoriesRoute.post(use: createHandler)
        categoriesRoute.get(":categoryID", use: getHandler)
        categoriesRoute.put(":categoryID", use: updateHandler)
        categoriesRoute.delete(":categoryID", use: deleteHandler)
        categoriesRoute.get(":categoryID", "acronyms", use: getAcronymsHandler)
    }
    
    @Sendable
    func getAllHandler(_ req: Request) async throws -> [Category] {
        try await Category.query(on: req.db).all()
    }
    
    @Sendable
    func createHandler(_ req: Request) async throws -> Category {
        let category = try req.content.decode(Category.self)
        try await category.save(on: req.db)
        return category
    }
    
    @Sendable
    func getHandler(_ req: Request) async throws -> Category {
        guard let category = try await Category.find(req.parameters.get("categoryID"), on: req.db) else {
            throw Abort(.notFound)
        }
        return category
    }

    @Sendable
    func updateHandler(_ req: Request) async throws -> Category {
        let category = try req.content.decode(Category.self)
        guard let existingCategory = try await Category.find(req.parameters.get("categoryID"), on: req.db) else {
            throw Abort(.notFound)
        }
        existingCategory.name = category.name
        try await existingCategory.update(on: req.db)
        return existingCategory
    }
    
    @Sendable
    func deleteHandler(_ req: Request) async throws -> HTTPStatus {
        guard let category = try await Category.find(req.parameters.get("categoryID"), on: req.db) else {
            throw Abort(.notFound)
        }
        try await category.delete(on: req.db)
        return .noContent
    }

    @Sendable
    func getAcronymsHandler(_ req: Request) async throws -> [Acronym] {
        guard let category = try await Category.find(req.parameters.get("categoryID"), on: req.db) else {
            throw Abort(.notFound)
        }
        return try await category.$acronyms.get(on: req.db)
    }
}